/**
 * Classification Module
 * @author James D Bloom 2003-03-12
 *
 * This module will have its code cleaned up (soon)
 * and more comments will be added so help
 * anyone who are tring to understand how is works
 */

package pipe.modules.classification;

import pipe.dataLayer.*;
import pipe.modules.Module;
import pipe.gui.*;
import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import javax.swing.border.*;
import java.util.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.event.MouseInputAdapter;
import java.util.ArrayList;
import java.util.Arrays;

public class Classification implements Module {

  // Main Frame
  private static final String MODULE_NAME = "Classification";
  private JFrame guiFrame = new JFrame("Petri-Net Classification Module");

  // Dimensions
  private final Toolkit mainToolKit = guiFrame.getToolkit(); // Get the window toolkit
  private final Dimension windowSize = mainToolKit.getScreenSize(); // Get screen size
  private final Point center = GraphicsEnvironment.getLocalGraphicsEnvironment().getCenterPoint(); // Get screen center
  private final int windowWidth = (windowSize.width*55)/100; // Window width in pixels
  private final int windowHeight = (windowSize.height*83)/100; // Window height in pixels
  private final int windowTopLeftCornerX = center.x-windowWidth/2; // Set window position
  private final int windowTopLeftCornerY = center.y-windowHeight/2; // Distance from top left corner of screen

  // Panes
  private JEditorPane codeEditorPane = new JEditorPane();
  private JScrollPane codeScrollPane = new JScrollPane(codeEditorPane);
  private JEditorPane consoleEditorPane = new JEditorPane();
  private JScrollPane consoleScrollPane = new JScrollPane(consoleEditorPane);
  private JScrollPane projectScrollPane = new JScrollPane(codeEditorPane);
  private JScrollPane classScrollPane = new JScrollPane(codeEditorPane);
  private JPanel guiResultsPanel = null;

  // Panels
  private JPanel mainPanel = new JPanel();
  private GridBagLayout gridbag = new GridBagLayout();
  private GridBagConstraints gridbagConstraints = new GridBagConstraints();

  // Text Fields
  private final int TEXT_FIELD_SIZE = 15;
  private JTextField sourceTextField = null;

  // Other
  private JButton sourceButton = new JButton();
  private JLabel sourceLabel = new JLabel();
  private final Font defaultLabel = null;
  private final Font smallDefaultLabel = null;
  private DataLayer sourcedataLayer = null;
  private DataLayer refdataLayer = null; // Storeage (maintain reference) for input dataLayer parameter;
  private File outputHTML = null;
  private FileWriter outputWriter = null;
  private Insets spacing = new Insets((windowHeight*15)/1000,(windowWidth*15)/1000,(windowHeight*15)/1000,(windowWidth*15)/1000);

  // Checkboxes
  private JCheckBox sourceCheckBox = new JCheckBox();
  private JCheckBox placeCheckBox = new JCheckBox();
  private JCheckBox placeIdCheckBox = new JCheckBox();
  private JCheckBox placeNameCheckBox = new JCheckBox();
  private JCheckBox placeMarkupCheckBox = new JCheckBox();
  private JCheckBox placePositionCheckBox = new JCheckBox();
  private JCheckBox transitionCheckBox = new JCheckBox();
  private JCheckBox transitionIdCheckBox = new JCheckBox();
  private JCheckBox transitionNameCheckBox = new JCheckBox();
  private JCheckBox transitionPositionCheckBox = new JCheckBox();
  private JCheckBox arcCheckBox = new JCheckBox();
  private JCheckBox arcSourceCheckBox = new JCheckBox();
  private JCheckBox arcTargetCheckBox = new JCheckBox();
  private JCheckBox arcStartCheckBox = new JCheckBox();
  private JCheckBox arcEndCheckBox = new JCheckBox();

  // Actions
  final static int NO_ACTION = -1;
  final static int CURRENT_NET = 0;
  final static int SOURCE_BROWSE = 1;
  final static int COMPARISON_BROWSE = 2;
  final static int CLASSIFY = 3;

  public void run(DataLayer pnmlData) {
//    warnUser(pnmlData.getURI(), guiFrame);
    Classification comparisonModule1 = new Classification(pnmlData);
  }

  public String getName() {
    return MODULE_NAME;
  }

  public Classification(DataLayer pnmlData) {
    sourcedataLayer = pnmlData;
    sourceTextField = new JTextField(sourcedataLayer.getURI(), TEXT_FIELD_SIZE);
    try {
      initGuiFrame();
      } catch (java.lang.Exception e) {}
  }

  public Classification() {
    sourceTextField = new JTextField(TEXT_FIELD_SIZE);
  }

  private void initGuiFrame() throws Exception {
    guiFrame.setBounds(windowTopLeftCornerX, windowTopLeftCornerY, windowWidth, windowHeight);
    guiFrame.setResizable(false);
    guiFrame.addMouseMotionListener(new MouseHandler());
    Container contentGuiPane = guiFrame.getContentPane();
    contentGuiPane.setLayout(new BorderLayout());
    JPanel guiPetriNetPanel = new JPanel();
    guiPetriNetPanel.addMouseMotionListener(new MouseHandler());
    guiPetriNetPanel.setToolTipText("Click on the \"Classify\" button to classify the Petri-Net");
    setUpPetriNetPanel(guiPetriNetPanel);
    guiPetriNetPanel.setVisible(true);
    contentGuiPane.add(guiPetriNetPanel, BorderLayout.NORTH);

    // Setup Classify Button
    JButton compareButton = new JButton("Classify");
    compareButton.addActionListener(new HandleControl(CLASSIFY));
    compareButton.setFont(new java.awt.Font("Dialog", 1, 18));
    compareButton.setToolTipText("Classify the Petri-Net");
    JPanel comparePane = new JPanel();
    comparePane.setLayout(gridbag);
    gridbagConstraints.insets = spacing;
    gridbag.setConstraints(compareButton, gridbagConstraints);
    comparePane.add(compareButton);
    contentGuiPane.add(comparePane, BorderLayout.SOUTH);
    setUpResultsPanel();
    guiFrame.setVisible(true);
  }

  private void setUpPetriNetPanel(JPanel panel){
    sourceCheckBox.addActionListener(new HandleControl(CURRENT_NET));
    sourceButton.addActionListener(new HandleControl(SOURCE_BROWSE));

    JPanel sourcePanel = new JPanel();
    sourcePanel.setLayout(new FlowLayout());

    // Setup Source Box
    sourceCheckBox.setText("use current net");
    sourceCheckBox.setSelected(true);

    sourceButton.setText("Browse");
    sourceButton.setEnabled(false);
    sourceLabel.setText("Source-Net URI:");
    sourceLabel.setEnabled(false);
    sourceTextField.setEnabled(false);
    sourcePanel.setLayout(new FlowLayout());

    sourcePanel.add(sourceCheckBox);
    sourcePanel.add(sourceLabel);
    sourcePanel.add(sourceTextField);
    sourcePanel.add(sourceButton);

    sourcePanel.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),"Source Petri-Net", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, defaultLabel, Color.BLACK));

    // Add to panel
    panel.setLayout(gridbag);
    gridbagConstraints.insets = spacing;
    gridbag.setConstraints(sourcePanel, gridbagConstraints);
    panel.add(sourcePanel);

    panel.setToolTipText("Specify the location of a Petri-Net");
  }

  private void setUpResultsPanel() throws IOException {
    guiResultsPanel = new JPanel();

    Color background = guiResultsPanel.getBackground();
    background.getRed();
    background.getBlue();
    background.getGreen();

    int red = background.getRed();
    int blue = background.getBlue();
    int green = background.getGreen();

    outputHTML = new File("ClassificationResults.html");

    outputHTML.createNewFile();
    outputWriter = new FileWriter(outputHTML);

    outputHTML.deleteOnExit();


    outputWriter.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<html>\n<head>");
    outputWriter.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=us-ascii\">\n<title></title>\n</head>");
    outputWriter.write("<body bgcolor=\"rgb(" + red + "," + green + "," + blue + ")\">\n");
    outputWriter.write("<table align=\"center\">\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\" colspan=\"2\"><img alt=\"Results\" src=\"Images/Results.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"ExtendedSimpleNet\" src=\"Images/ExtendedSimpleNet.png\"></th>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Unknown\" src=\"Images/Unknown.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"SimpleNet\" src=\"Images/SimpleNet.png\"></th>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Unknown\" src=\"Images/Unknown.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"ExtendedFreeChoiceNet\" src=\"Images/ExtendedFreeChoiceNet.png\"></th>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Unknown\" src=\"Images/Unknown.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"FreeChoiceNet\" src=\"Images/FreeChoiceNet.png\"></th>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Unknown\" src=\"Images/Unknown.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"MarkedGraph\" src=\"Images/MarkedGraph.png\"></th>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Unknown\" src=\"Images/Unknown.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"StateMachine\" src=\"Images/StateMachine.png\"></th>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Unknown\" src=\"Images/Unknown.png\"></th></tr></table>\n");
    outputWriter.write("</body>\n</html>\n");
    outputWriter.close();
    displayResults();
    outputHTML.delete();
  }

  private void warnUser(String message, JFrame frame) {
    JOptionPane.showMessageDialog(frame, message, "Petri-Net Classification Module", JOptionPane.CLOSED_OPTION);
  }

  private void classifyPetriNets() throws IOException {
    guiResultsPanel.setVisible(false);
    Color background = guiResultsPanel.getBackground();
    background.getRed();
    background.getBlue();
    background.getGreen();

    int red = background.getRed();
    int blue = background.getBlue();
    int green = background.getGreen();

    outputHTML = new File("ClassificationResults.html");
    outputHTML.delete();
    outputHTML.createNewFile();
    outputWriter = new FileWriter(outputHTML);

    outputHTML.deleteOnExit();

    outputWriter.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<html>\n<head>");
    outputWriter.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=us-ascii\">\n<title></title>\n</head>");
    outputWriter.write("<body bgcolor=\"rgb(" + red + "," + green + "," + blue + ")\">\n");
    outputWriter.write("<table align=\"center\">\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\" colspan=\"2\"><img alt=\"Results\" src=\"Images/Results.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"ExtendedSimpleNet\" src=\"Images/ExtendedSimpleNet.png\"></th>\n");
    if(extendedSimpleNet(sourcedataLayer))
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Yes\" src=\"Images/Yes.png\"></th></tr>\n");
    else
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"No\" src=\"Images/No.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"SimpleNet\" src=\"Images/SimpleNet.png\"></th>\n");
    if(simpleNet(sourcedataLayer))
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Yes\" src=\"Images/Yes.png\"></th></tr>\n");
    else
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"No\" src=\"Images/No.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"ExtendedFreeChoiceNet\" src=\"Images/ExtendedFreeChoiceNet.png\"></th>\n");
    if(extendedFreeChoiceNet(sourcedataLayer))
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Yes\" src=\"Images/Yes.png\"></th></tr>\n");
    else
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"No\" src=\"Images/No.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"FreeChoiceNet\" src=\"Images/FreeChoiceNet.png\"></th>\n");
    if(freeChoiceNet(sourcedataLayer))
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Yes\" src=\"Images/Yes.png\"></th></tr>\n");
    else
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"No\" src=\"Images/No.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"MarkedGraph\" src=\"Images/MarkedGraph.png\"></th>\n");
    if(markedGraph(sourcedataLayer))
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Yes\" src=\"Images/Yes.png\"></th></tr>\n");
    else
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"No\" src=\"Images/No.png\"></th></tr>\n");
    outputWriter.write("<tr>\n");
    outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"StateMachine\" src=\"Images/StateMachine.png\"></th>\n");
    if(stateMachine(sourcedataLayer))
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"Yes\" src=\"Images/Yes.png\"></th></tr>\n");
    else
      outputWriter.write("<th nowrap=\"nowrap\"><img alt=\"No\" src=\"Images/No.png\"></th></tr>\n");
    outputWriter.write("</body>\n</html>\n");
    outputWriter.close();
    displayResults();
    guiResultsPanel.setVisible(true);
    outputHTML.delete();
  }

  private void displayResults() throws IOException {
    Color background = guiResultsPanel.getBackground();
    background.getRed();
    background.getBlue();
    background.getGreen();

    int red = background.getRed();
    int blue = background.getBlue();
    int green = background.getGreen();

    JEditorPane editorPane = new JEditorPane();
    editorPane.setBackground(background);
    editorPane.setPage("file:" + outputHTML.getAbsolutePath());
    editorPane.setEditable(false);
    JScrollPane scrollPane = new JScrollPane(editorPane);

    int w = windowWidth*1/2;
    int h = windowHeight*3/4;
    int x = center.x-w/2;
    int y = center.y-h/2;
    editorPane.setBounds(x, y, w, h);
    Dimension dimension = new Dimension(w, h);
    editorPane.setMinimumSize(dimension);
    scrollPane.setMinimumSize(dimension);
    guiResultsPanel.setMinimumSize(dimension);
    guiResultsPanel = new JPanel();
    guiResultsPanel.addMouseMotionListener(new MouseHandler());
    guiResultsPanel.add(editorPane);
    guiResultsPanel.setVisible(true);
    guiResultsPanel.setBorder(BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)));
    JPanel guiResultsOuterPanel = new JPanel();
    guiResultsOuterPanel.setLayout(gridbag);
    gridbagConstraints.insets = spacing;
    gridbag.setConstraints(guiResultsPanel, gridbagConstraints);
    guiResultsOuterPanel.add(guiResultsPanel);
    guiFrame.getContentPane().add(guiResultsOuterPanel, BorderLayout.CENTER);
  }
/*
  private boolean extendedSimpleNet = true;
  private boolean simpleNet = true;
  private boolean extendedFreeChoiceNet = true;
  private boolean freeChoiceNet = true;
  private boolean markedGraph = true;
  private boolean stateMachine = true;
*/
  private boolean stateMachine(DataLayer pnmlData){
    Transition[] transitions = pnmlData.getTransitions();
    if(transitions != null) {

      int transitionNo = 0;
      for(transitionNo = 0 ; transitionNo < transitions.length ; transitionNo++){
        if (sumTransitionBackwards(transitionNo) > 1 || sumTransitionForwards(transitionNo) > 1) {
          if (sumTransitionBackwards(transitionNo) == -1 || sumTransitionForwards(transitionNo) == -1) {
            warnUser("Error in Transition", guiFrame);
          }
          return false;
        }
      }
    }
    return true;
  }

  private boolean markedGraph(DataLayer pnmlData){
    Place[] places = pnmlData.getPlaces();
    if(places != null) {

      int placeNo = 0;
      for(placeNo = 0 ; placeNo < places.length ; placeNo++){
        if (sumPlaceBackwards(placeNo) > 1 || sumPlaceForwards(placeNo) > 1) {
          if (sumPlaceBackwards(placeNo) == -1 || sumPlaceForwards(placeNo) == -1) {
            warnUser("Error in Place", guiFrame);
          }
          return false;
        }
      }
    }
    return true;
  }

  private boolean freeChoiceNet(DataLayer pnmlData){
    Place[] places = pnmlData.getPlaces();
    if(places != null) {
      int placeNo = 0;
      int placeDashNo = 0;
      for(placeNo = 0 ; placeNo < places.length ; placeNo++){
        for(placeDashNo = 0 ; placeDashNo < places.length ; placeDashNo++){
          if(placeDashNo != placeNo) {
            int[] placeSet = forwardsPlaceSet(placeNo);
            int[] placeDashSet = forwardsPlaceSet(placeDashNo);
            if(unionBetweenSets(placeSet, placeDashSet) == false &&
               (sumPlaceForwards(placeNo) > 1 || sumPlaceForwards(placeDashNo) > 1)) {
              return false;
            }
          }
        }
      }
    }
    return true;
  }

  private boolean extendedFreeChoiceNet(DataLayer pnmlData){
    if(freeChoiceNet(pnmlData)) {
      return true;
    }
    Place[] places = pnmlData.getPlaces();
    if(places != null) {
      int placeNo = 0;
      int placeDashNo = 0;
      for(placeNo = 0 ; placeNo < places.length ; placeNo++){
        for(placeDashNo = 0 ; placeDashNo < places.length ; placeDashNo++){
          if(placeDashNo != placeNo) {
            int[] placeSet = backwardsPlaceSet(placeNo);
            int[] placeDashSet = backwardsPlaceSet(placeDashNo);
            if(unionBetweenSets(placeSet, placeDashSet)) {
              if(Arrays.equals(placeSet, placeDashSet)){
                return true;
              }
            }
          }
        }
      }
    }
    return false;
  }

  private boolean simpleNet(DataLayer pnmlData){
    Place[] places = pnmlData.getPlaces();
    if(places != null) {
      int placeNo = 0;
      int placeDashNo = 0;
      for(placeNo = 0 ; placeNo < places.length ; placeNo++){
        for(placeDashNo = 0 ; placeDashNo < places.length ; placeDashNo++){
          if(placeDashNo != placeNo) {
            int[] placeSet = forwardsPlaceSet(placeNo);
            int[] placeDashSet = forwardsPlaceSet(placeDashNo);
            if(sumPlaceForwards(placeNo) > 1 &&
               sumPlaceForwards(placeDashNo) > 1 &&
               unionBetweenSets(placeSet, placeDashSet) == false) {
              return false;
            }
          }
        }
      }
    }
    return true;
  }

  private boolean extendedSimpleNet(DataLayer pnmlData){
    if(simpleNet(pnmlData)) {
      return true;
    }
    Place[] places = pnmlData.getPlaces();
    if(places != null) {
      int placeNo = 0;
      int placeDashNo = 0;
      for(placeNo = 0 ; placeNo < places.length ; placeNo++){
        for(placeDashNo = 0 ; placeDashNo < places.length ; placeDashNo++){
          if(placeDashNo != placeNo) {
            int[] placeSet = forwardsPlaceSet(placeNo);
            int[] placeDashSet = forwardsPlaceSet(placeDashNo);
            if(isSubset(placeSet, placeDashSet) == false) {
              return true;
            }
          }
        }
      }
    }
    return false;
/*
    Place[] places = pnmlData.getPlaces();
    if(places != null) {
      int placeNo = 0;
      int placeDashNo = 0;
      for(placeNo = 0 ; placeNo < places.length ; placeNo++){
        for(placeDashNo = 0 ; placeDashNo < places.length ; placeDashNo++){
          if(placeDashNo != placeNo) {
            int[] placeSet = forwardsPlaceSet(placeNo);
            int[] placeDashSet = forwardsPlaceSet(placeDashNo);
            if(isSubSetRelationship(placeSet, placeDashSet) &&
               unionBetweenSets(placeSet, placeDashSet) == false) {
              return false;
            }
          }
        }
      }
    }
    return true;
*/
  }

  private int sumPlaceBackwards(int PlaceNo) {
    int[][] backwards = sourcedataLayer.getFowardsIncidenceMatrix();
    int preset = 0;

    if(PlaceNo < backwards.length){
      for(int TransitionNo = 0 ; TransitionNo < backwards[PlaceNo].length ; TransitionNo++) {
        if(backwards[PlaceNo][TransitionNo] != 0) {
          preset++;
        }
      }
    } else {
      return -1;
    }

    return preset;
  }

  private int sumPlaceForwards(int PlaceNo) {
    int[][] forwards = sourcedataLayer.getBackwardsIncidenceMatrix();
    int postset = 0;

    if(PlaceNo < forwards.length){

      for(int TransitionNo = 0 ; TransitionNo < forwards[PlaceNo].length ; TransitionNo++) {
        if(forwards[PlaceNo][TransitionNo] != 0) {
          postset++;
        }
      }

    } else {
      return -1;
    }
    return postset;
  }

  private int sumTransitionBackwards(int TransitionNo) {
    int[][] backwards = sourcedataLayer.getBackwardsIncidenceMatrix();
    int preset = 0;

    for(int PlaceNo = 0 ; PlaceNo < backwards.length ; PlaceNo++) {

      if(TransitionNo < backwards[PlaceNo].length){
        if(backwards[PlaceNo][TransitionNo] != 0) {
          preset++;
        }
      } else {
        return -1;
      }

    }

    return preset;
  }

  private int sumTransitionForwards(int TransitionNo) {
    int[][] forwards = sourcedataLayer.getFowardsIncidenceMatrix();
    int postset = 0;

    for(int PlaceNo = 0 ; PlaceNo < forwards.length ; PlaceNo++) {

      if(TransitionNo < forwards[PlaceNo].length){
        if(forwards[PlaceNo][TransitionNo] != 0) {
          postset++;
        }
      } else {
        return -1;
      }

    }

    return postset;
  }

  private int[] backwardsPlaceSet(int PlaceNo){
    int[][] backwards = sourcedataLayer.getBackwardsIncidenceMatrix();
    ArrayList presetArrayList = new ArrayList();

    for(int TransitionNo = 0 ; TransitionNo < backwards[PlaceNo].length ; TransitionNo++) {
      if(backwards[PlaceNo][TransitionNo] != 0) {
        presetArrayList.add(new Integer(TransitionNo));
      }
    }

    int[] preset = new int[presetArrayList.size()];

    for(int presetPosition = 0 ; presetPosition < preset.length ; presetPosition++) {
      preset[presetPosition] = ((Integer)presetArrayList.get(presetPosition)).intValue();
    }

    return preset;
  }

  private int[] forwardsPlaceSet(int PlaceNo){
    int[][] forwards = sourcedataLayer.getFowardsIncidenceMatrix();
    ArrayList postsetArrayList = new ArrayList();

    for(int TransitionNo = 0 ; TransitionNo < forwards[PlaceNo].length ; TransitionNo++) {
      if(forwards[PlaceNo][TransitionNo] != 0) {
        postsetArrayList.add(new Integer(TransitionNo));
      }
    }

    int[] postset = new int[postsetArrayList.size()];

    for(int postsetPosition = 0 ; postsetPosition < postset.length ; postsetPosition++) {
      postset[postsetPosition] = ((Integer)postsetArrayList.get(postsetPosition)).intValue();
    }

    return postset;
  }

  private boolean unionBetweenSets(int[] setOne, int[] setTwo) {
    for(int onePosition = 0 ; onePosition < setOne.length ; onePosition++) {
      for(int twoPosition = 0 ; twoPosition < setTwo.length ; twoPosition++) {
        if(setOne[onePosition] == setTwo[twoPosition]) {
          return true;
        }
      }
    }
    return false;
  }

  public static boolean isSubset(int[] A, int[] B){

    boolean isSubsetOf = false;

    // first you must eliminate all dublicate values from the sets, so run through each set
    // and replace all dublicates with an inpossible (flag) value, e.g. -1
    int[] AA;
    AA = reduce(A);
    int[] BB;
    BB = reduce(B);

    // then you must determine which set has the smallest cardinality (number of
    // distinct values, this is why we did the above operation)
    int AAcard = cardinality(AA);
    int BBcard = cardinality(BB);

    // then look for the set with smaller cardinality being subset of the set with greater cardinality

    // make sure 1st agument has smaller cardinality than 2nd
    if (AAcard< BBcard)
      isSubsetOf = findSubset(AA, BB);
    else
      isSubsetOf = findSubset(BB, AA);

    return isSubsetOf;
  }

  // eliminate all dublicate values from the set, so run through the set
  // and replace all dublicates with an inpossible (flag) value, e.g. -1
  public static int[] reduce(int[] X){

    for (int i=0; i < X.length; i++){
      for (int j = 0; j < i; j++){ // check values before current index
        if (X[i] == X[j]) {// value already exists in the set
          X[i] = -1; // flag dublicate
          break;  // and break the inner loop
        }
      }
    }

    return X;
  }

  // determine set cardinality (number of distinct values)
  public static int cardinality(int[] X){
    int c = 0; // carinality
    for (int i = 0; i < X.length; i++)
      if (X[i] != -1)
        c++;

    return c;
  }

  // A has smaller cardinality than B, so only A can be subset of B and not the opposite
  public static boolean findSubset(int[] A, int[] B){
    boolean isSubset = true;
    boolean elementExists;

    // loop through A and see if each of its elements is contained in B
    // if at least one is not contained, then it is not a subset. watch for flags -1
    for (int i = 0; i < A.length; i++){
      elementExists = false;
      for (int j = 0; j < B.length; j++){
        if (A[i] == B[j] && A[i] != -1){ // ith element of A exists in B
          elementExists = true;
          break;
        }
      }
      // test for particular ith element of A in all of B
      if (!elementExists)
        return false;
    }

    return isSubset;
  }


  private boolean isSubSetRelationship(int[] setOne, int[] setTwo) {
    int onePosition = 0;
    int twoPosition = 0;
    if(setOne.length > setTwo.length) {
      for( ; twoPosition < setTwo.length ; twoPosition++) {
        if(search(setTwo[twoPosition], setOne) < 0) {
          return false;
        }
      }
      return true;
    } else if (setOne.length <= setTwo.length) {
      for( ; onePosition < setOne.length ; onePosition++) {
        if(search(setOne[onePosition], setTwo) < 0) {
          return false;
        }
      }
      return true;
    }
    return false;
  }

  int search(int value, int[] array) {
/*
    if(array != null) {
      int first = 0;
      int last = array.length;
      quickSort(array, first, last);
      return binarySearch(value, array, first, last);
    }
    return -1;
*/
    return Arrays.binarySearch(array, value);
  }

  void quickSort(int[] array, int first, int last)
  {
    if(array != null) {
      int left = first;
      int right = last;
      int pivot = array[(left + right)/2];

      do {
        while(right > 0 && array[right] > pivot) {
          right--;
        }
        while(right < array.length && array[right] < pivot) {
          right++;
        }
        if(right < array.length && left > 0 && left <= right) {
          int temp = array[left];
          array[left] = array[right];
          array[right] = temp;
          left++;
          right--;
        }

      } while (right >= left);
      if(first < right)
        quickSort(array, first, right);
      if(left < last)
        quickSort(array, left, last);
    }
  }

  int binarySearch(int value, int[] array, int first, int last)
  {
    int midPoint = (first + last)/2;

    if(first > last) {
      return -1;
    }
    else {
      if(array[midPoint] == value)
        return midPoint;
      else if(array[midPoint] > value)
        return binarySearch(value, array, first, midPoint -1);
      else
        return binarySearch(value, array, midPoint + 1, last);
    }
  }


  class MouseHandler extends MouseInputAdapter {

    public void mouseMoved(MouseEvent e) {
      guiResultsPanel.setVisible(false);
      guiResultsPanel.setVisible(true);
    }
  }

  // Class defining a handler for a control button
  class HandleControl implements ActionListener {
    private int buttonID;

    // Constructor
    public HandleControl(int buttonID) {
      this.buttonID = buttonID; // Store the button ID
    }

    private String openFile(File file, String title, JTextField textField) {
      int fileChooserReturnVal = JFileChooser.ERROR_OPTION;
      JFileChooser fileChooser = null;
      if(sourcedataLayer.getURI() != null)
        fileChooser = new JFileChooser(sourcedataLayer.getURI());
      else
        fileChooser = new JFileChooser();

      String explain = "Please select " + title;

      fileChooser.setToolTipText(explain);
      fileChooser.setName(explain);
      fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
      fileChooser.setDialogTitle(explain);
      fileChooser.setFileSelectionMode(fileChooser.FILES_ONLY);
      ExtensionFilter netFilter = new ExtensionFilter(".xml","Petri-Net files (*.xml)");

      fileChooser.setAcceptAllFileFilterUsed(false); // Remove 'all files'filter
      fileChooser.addChoosableFileFilter(netFilter); // add Filter
      fileChooser.setFileFilter(netFilter); // and select it
      fileChooser.rescanCurrentDirectory();

      if(file != null)
        fileChooser.setSelectedFile(file);

      fileChooserReturnVal = fileChooser.showOpenDialog(guiFrame);

      if (fileChooserReturnVal == JFileChooser.APPROVE_OPTION) {

        file = fileChooser.getSelectedFile();

        if (file.getName().endsWith(".xml")) {

          try {

            if(file != null)
              sourcedataLayer = new DataLayer(file);

          } catch (java.io.IOException ex) {
            warnUser("java.io.IOException " + ex + " File Access Error with " + title, guiFrame);
          } catch (org.xml.sax.SAXException ex) {
            warnUser("org.xml.sax.SAXException " + ex + " File Access Error with " + title, guiFrame);
          } catch (javax.xml.transform.TransformerException  ex) {
            warnUser("javax.xml.transform.TransformerException " + ex + " File Access Error with " + title, guiFrame);
          } catch (javax.xml.parsers.ParserConfigurationException  ex) {
            warnUser("javax.xml.parsers.ParserConfigurationException " + ex + " File Access Error with " + title, guiFrame);
          }

        } else {
          int returnVal = JOptionPane.showConfirmDialog(guiFrame, "Incorrect file type selected do you want to try again?", "Please select a Petri-Net File (*.xml)", JOptionPane.YES_NO_OPTION, JOptionPane.PLAIN_MESSAGE);
          if(returnVal == JOptionPane.YES_OPTION) {
            textField.setText(null);
            file = null;
            return openFile(file, title, textField);
          }
        }

      }

      if(file == null) return null;

      return file.getName();
    }

    // Handle button click
    public void actionPerformed(ActionEvent e) {
      File sourceFile = null;

      switch(buttonID) {
        case NO_ACTION:
          break;
        case CURRENT_NET:
          guiResultsPanel.setVisible(false);
          guiResultsPanel.setVisible(true);
          if(sourceCheckBox.isSelected()) {
            sourceButton.setEnabled(false);
            sourceLabel.setEnabled(false);
            sourceTextField.setEnabled(false);
            sourcedataLayer = refdataLayer;
          } else {
            sourceButton.setEnabled(true);
            sourceLabel.setEnabled(true);
            sourceTextField.setEnabled(true);
            refdataLayer = sourcedataLayer;
          }
          sourceTextField.setText(sourcedataLayer.getURI());
          break;
        case SOURCE_BROWSE:
          guiResultsPanel.setVisible(false);
          guiResultsPanel.removeAll();
          sourceTextField.setText(openFile(sourceFile,  "the Source Petri-Net file", sourceTextField));
          try {
            setUpResultsPanel();
            } catch (IOException ioe) { warnUser("Internal File Error", guiFrame); }
            break;
          case CLASSIFY:
            try {
              classifyPetriNets();
              } catch (IOException ioe) { warnUser("Internal File Error", guiFrame); }
              guiResultsPanel.setVisible(false);
              guiResultsPanel.setVisible(true);
              break;
      }
    }
  }
}